@@ -7,7 +7,7 @@ package HTML::Mason::Compiler::ToObject;
 use strict;
 use warnings;
 
-use Params::Validate qw(SCALAR validate);
+use Params::Validate qw(BOOLEAN SCALAR validate);
 use HTML::Mason::Tools qw(taint_is_on);
 
 use HTML::Mason::Compiler;
@@ -50,6 +50,10 @@ BEGIN
          { parse => 'string', type => SCALAR, default => 'auto',
            regex => qr/^(?:always|auto|never)$/,
            descr => "Whether or not to create the %ARGS hash" },
+
+         named_component_subs =>
+         { parse => 'boolean', type => BOOLEAN, default => 0,
+           descr => "Whether to use named subroutines for component code" },
         );
 }
 
@@ -58,6 +62,7 @@ use HTML::Mason::MethodMaker
                     qw( comp_class
                         define_args_hash
                         in_package
+                        named_component_subs
                         postamble
                         preamble
                         subcomp_class
@@ -114,6 +119,7 @@ sub compile_to_file
     $self->compile( comp_source => $source->comp_source_ref,
                     name => $source->friendly_name,
                     comp_class => $source->comp_class,
+                    comp_path => $source->comp_path,
                     fh => $fh );
 
     close $fh 
@@ -176,15 +182,33 @@ sub compiled_component
         $subs{main} = $params->{code};
         $params->{code} = "sub {\nHTML::Mason::Request->instance->call_dynamic( 'main', \@_ )\n}";
 
+        my $named_subs = '';
+        my %named_subs = $self->_named_subs_hash;
+        while ( my ( $name, $body ) = each %named_subs )
+        {
+            $named_subs .= '*' . $name . " = sub {\n" . $body . "\n};\n\n";
+        }
+
         $params->{dynamic_subs_init} =
             join '', ( "sub {\n",
                        $self->_set_request,
                        $self->_blocks('shared'),
+                       $named_subs,
                        "return {\n",
                        map( "'$_' => $subs{$_},\n", sort keys %subs ),
                        "\n}\n}"
                      );
     }
+    else
+    {
+        my %named_subs = $self->_named_subs_hash;
+        while ( my ( $name, $body ) = each %named_subs )
+        {
+            $self->_output_chunk( $p{fh}, \$obj_text,
+                                  "sub $name {\n" . $body . "\n}\n"
+                                );
+        }
+    }
 
     $self->_output_chunk($p{fh}, \$obj_text, $self->_subcomponents_footer);
     $self->_output_chunk($p{fh}, \$obj_text, $self->_methods_footer);
@@ -198,6 +222,46 @@ sub compiled_component
     return \$obj_text;
 }
 
+sub _named_subs_hash
+{
+    my $self = shift;
+
+    return unless $self->named_component_subs;
+
+    my %subs;
+    $subs{ $self->_sub_name } = $self->_body;
+
+    while ( my ( $name, $params ) =
+            each %{ $self->{current_compile}{compiled_def} } )
+    {
+        $subs{ $self->_sub_name( 'def', $name ) } = $params->{body};
+    }
+
+    while ( my ( $name, $params ) =
+            each %{ $self->{current_compile}{compiled_method} } )
+    {
+        $subs{ $self->_sub_name( 'method', $name ) } = $params->{body};
+    }
+
+    return %subs;
+}
+
+sub _sub_name
+{
+    my $self = shift;
+
+    return join '_', $self->_escape_sub_name_part( $self->{comp_path}, @_ );
+}
+
+sub _escape_sub_name_part
+{
+    my $self = shift;
+
+    return map { my $part = $_;
+                 $part =~ s/([^\w_])/'_' . sprintf( '%x', ord $1 )/ge;
+                 $part; } @_;
+}
+
 sub _compile_subcomponents
 {
     my $self = shift;
@@ -279,7 +343,8 @@ sub _constructor
     my ($self, $class, $params) = @_;
 
     return ("${class}->new(\n",
-            map( {("'$_' => ", $params->{$_}, ",\n")} sort keys %$params ),
+            map( {("'$_' => ", $params->{$_}, ",\n")}
+                 sort grep { $_ ne 'body' } keys %$params ),
             "\n)\n",
            );
 }
@@ -288,8 +353,22 @@ sub _component_params
 {
     my $self = shift;
 
-    my %params = ( code => join ( '', "sub {\n", $self->_body, "}" ),
-                 );
+    my %params;
+
+    if ( $self->named_component_subs )
+    {
+        $params{code} =
+            '\\&' .
+            $self->_sub_name
+                ( grep { defined }
+                  @{ $self->{current_compile}{in_named_block} }
+                  { 'type', 'name' } );
+        $params{body} = $self->_body;
+    }
+    else
+    {
+        $params{code} = join '', "sub {\n", $self->_body, "}";
+    }
 
     $params{flags} = join '', "{\n", $self->_flags, "\n}"
         if keys %{ $self->{current_compile}{flags} };
@@ -438,8 +517,11 @@ EOF
             $coerce = $arg_in_array;
         }
 
-        push @assign, "#line $_->{line} $_->{file}\n"
-            if defined $_->{line} && defined $_->{file} && $self->use_source_line_numbers;
+        if ( defined $_->{line} && defined $_->{file} && $self->use_source_line_numbers )
+        {
+            my $file = $self->_escape_filename( $_->{file} );
+            push @assign, qq{#line $_->{line} "$file"\n};
+        }
 
         if ( defined $_->{default} )
         {
@@ -558,7 +640,11 @@ HTML::Mason::Compiler::ToObject - A Compiler subclass that generates Mason objec
 
   my $compiler = HTML::Mason::Compiler::ToObject->new;
 
-  my $object_code = $compiler->compile( comp_source => $source, name => $comp_name );
+  my $object_code =
+      $compiler->compile( comp_source => $source,
+                          name        => $comp_name,
+                          comp_path   => $comp_path,
+                        );
 
 =head1 DESCRIPTION
 
@@ -604,6 +690,13 @@ C<$m> in postamble code.
 True or false, default is true. Indicates whether or not a given
 component should C<use strict>.
 
+=item named_component_subs
+
+When compiling a component, use uniquely named subroutines for the a
+component's body, subcomponents, and methods. Doing this allows you to
+effectively profile Mason components. Without this, all components
+simply show up as __ANON__ or something similar in the profiler.
+
 =item define_args_hash
 
 One of "always", "auto", or "never".  This determines whether or not
@@ -634,12 +727,14 @@ and as such has a very limited public API.
 
 =over
 
-=item compile (comp_source => $source, name => $name, comp_class = $comp_class)
+=item compile(...)
 
 This method will take component source and return the compiled object
-code for that source.  The C<comp_source> and C<name> parameters are
-optional.  The C<comp_class> can be used to change the component class
-for this one comonent.
+code for that source. See L<HTML::Mason::Compiler/compile(...)> for
+details on this method.
+
+This subclass also accepts a C<comp_class> parameter, allowing you to
+override the class into which the component is compiled.
 
 =back
 
